@@ -15,17 +15,20 @@ import sys
 import tarfile
 from getpass import getpass
 from typing import Optional
+from shutil import copyfile
 
 import click
 import wrapt
 import yaml
-from kodexa.platform.client import DEFAULT_COLUMNS
+from functional import seq
+from kodexa.model import ModelContentMetadata
+from kodexa.platform.client import DEFAULT_COLUMNS, ModelStoreEndpoint
 from rich import print
 
 logging.root.addHandler(logging.StreamHandler(sys.stdout))
 
 from kodexa import KodexaClient
-from kodexa.platform.kodexa import ExtensionHelper, KodexaPlatform
+from kodexa.platform.kodexa import KodexaPlatform
 
 LOGGING_LEVELS = {
     0: logging.NOTSET,
@@ -72,6 +75,32 @@ def merge(a, b, path=None):
     return a
 
 
+class MetadataHelper:
+    """ """
+
+    @staticmethod
+    def load_metadata(path, filename: Optional[str]) -> dict:
+
+        if filename is not None:
+            dharma_metadata_file = open(os.path.join(path, filename))
+            if filename.endswith('.json'):
+                dharma_metadata = json.loads(dharma_metadata_file.read())
+            elif filename.endswith('.yml'):
+                dharma_metadata = yaml.safe_load(dharma_metadata_file.read())
+        elif os.path.exists(os.path.join(path, 'dharma.json')):
+            dharma_metadata_file = open(os.path.join(path, 'dharma.json'))
+            dharma_metadata = json.loads(dharma_metadata_file.read())
+        elif os.path.exists(os.path.join(path, 'dharma.yml')):
+            dharma_metadata_file = open(os.path.join(path, 'dharma.yml'))
+            dharma_metadata = yaml.safe_load(dharma_metadata_file.read())
+        elif os.path.exists(os.path.join(path, 'kodexa.yml')):
+            dharma_metadata_file = open(os.path.join(path, 'kodexa.yml'))
+            dharma_metadata = yaml.safe_load(dharma_metadata_file.read())
+        else:
+            raise Exception("Unable to find a kodexa.yml file describing your extension")
+        return dharma_metadata
+
+
 # Change the options to below to suit the actual options for your task (or
 # tasks).
 @click.group()
@@ -171,8 +200,10 @@ def upload(_: Info, ref: str, paths: list[str], token: str, url: str):
 @click.option('--token', default=KodexaPlatform.get_access_token(), help='Access token')
 @click.option('--format', default=None, help='The format to input if from stdin (json, yaml)')
 @click.option('--overlay', default=None, help='A JSON or YAML file that will overlay the metadata')
+@click.argument('files', nargs=-1)
 @pass_info
-def deploy(_: Info, org: Optional[str], file: str, url: str, token: str, format=None, update: bool = False,
+def deploy(_: Info, org: Optional[str], file: str, files: list[str], url: str, token: str, format=None,
+           update: bool = False,
            version=None, overlay: Optional[str] = None, slug=None):
     """
     Deploy a component to a Kodexa platform instance from a file or stdin
@@ -181,7 +212,67 @@ def deploy(_: Info, org: Optional[str], file: str, url: str, token: str, format=
     client = KodexaClient(access_token=token, url=url)
 
     obj = None
-    if file is None:
+
+    def deploy_obj(obj):
+        if 'deployed' in obj:
+            del obj['deployed']
+
+        overlay_obj = None
+
+        if overlay is not None:
+            print("Reading overlay")
+            if overlay.endswith('yaml') or overlay.endswith('yml'):
+                overlay_obj = yaml.safe_load(sys.stdin.read())
+            elif overlay.endswith('json'):
+                overlay_obj = json.loads(sys.stdin.read())
+            else:
+                raise Exception("Unable to determine the format of the overlay file, must be .json or .yml/.yaml")
+
+        if isinstance(obj, list):
+            print(f"Found {len(obj)} components")
+            for o in obj:
+
+                if overlay_obj:
+                    o = merge(o, overlay_obj)
+
+                component = client.deserialize(o)
+                if org is not None:
+                    component.org_slug = org
+                print(f"Deploying component {component.slug}:{component.version}")
+                component.deploy(update=update)
+
+        else:
+
+            if overlay_obj:
+                obj = merge(obj, overlay_obj)
+
+            component = client.deserialize(obj)
+
+            if version is not None:
+                component.version = version
+            if slug is not None:
+                component.slug = slug
+            if org is not None:
+                component.org_slug = org
+            print(f"Deploying component {component.slug}:{component.version}")
+            log_details = component.deploy(update=update)
+            for log_detail in log_details:
+                print(log_detail)
+
+    if files is not None:
+        print("Reading from files", files)
+        obj = {}
+        for file in files:
+            with open(file, 'r') as f:
+                if file.lower().endswith('.json'):
+                    obj.update(json.load(f))
+                elif file.lower().endswith('.yaml') or file.lower().endswith('.yml'):
+                    obj.update(yaml.safe_load(f))
+                else:
+                    raise Exception("Unsupported file type")
+
+                deploy_obj(obj)
+    elif file is None:
         print("Reading from stdin")
         if format == 'yaml' or format == 'yml':
             obj = yaml.safe_load(sys.stdin.read())
@@ -189,6 +280,8 @@ def deploy(_: Info, org: Optional[str], file: str, url: str, token: str, format=
             obj = json.loads(sys.stdin.read())
         else:
             raise Exception("You must provide a format if using stdin")
+
+        deploy_obj(obj)
     else:
         print("Reading from file", file)
         with open(file, 'r') as f:
@@ -199,50 +292,8 @@ def deploy(_: Info, org: Optional[str], file: str, url: str, token: str, format=
             else:
                 raise Exception("Unsupported file type")
 
-    if 'deployed' in obj:
-        del obj['deployed']
+            deploy_obj(obj)
 
-    overlay_obj = None
-
-    if overlay is not None:
-        print("Reading overlay")
-        if overlay.endswith('yaml') or overlay.endswith('yml'):
-            overlay_obj = yaml.safe_load(sys.stdin.read())
-        elif overlay.endswith('json'):
-            overlay_obj = json.loads(sys.stdin.read())
-        else:
-            raise Exception("Unable to determine the format of the overlay file, must be .json or .yml/.yaml")
-
-    if isinstance(obj, list):
-        print(f"Found {len(obj)} components")
-        for o in obj:
-
-            if overlay_obj:
-                o = merge(o, overlay_obj)
-
-            component = client.deserialize(o)
-            if org is not None:
-                component.org_slug = org
-            print(f"Deploying component {component.slug}:{component.version}")
-            component.deploy(update=update)
-
-    else:
-
-        if overlay_obj:
-            obj = merge(obj, overlay_obj)
-
-        component = client.deserialize(obj)
-
-        if version is not None:
-            component.version = version
-        if slug is not None:
-            component.slug = slug
-        if org is not None:
-            component.org_slug = org
-        print(f"Deploying component {component.slug}:{component.version}")
-        log_details = component.deploy(update=update)
-        for log_detail in log_details:
-            print(log_detail)
     print("Deployed :tada:")
 
 
@@ -356,7 +407,7 @@ def print_object_table(object_metadata, objects_endpoint, query, page, pagesize,
     console = Console()
     console.print(table)
     console.print(
-        f"Page [bold]{page_of_object_endpoints.number}[/bold] of [bold]{page_of_object_endpoints.total_pages}[/bold] "
+        f"Page [bold]{page_of_object_endpoints.number + 1}[/bold] of [bold]{page_of_object_endpoints.total_pages}[/bold] "
         f"(total of {page_of_object_endpoints.total_elements} objects)")
 
 
@@ -489,13 +540,16 @@ def platform(_: Info, python: bool):
 
     if platform_url is not None:
         print(f"Kodexa URL: {KodexaPlatform.get_url()}")
-        print(f"Access Token: {KodexaPlatform.get_access_token()}")
         kodexa_version = KodexaPlatform.get_server_info()
+        print(f"Environment: {kodexa_version['environment']}")
         print(f"Version: {kodexa_version['version']}")
         print(f"Release: {kodexa_version['release']}")
+        print(f"Extension Packs:")
+        for ep in seq(kodexa_version['extensionPacks']).map(lambda x: x['ref']).list():
+            print(f"  {ep}")
         if python:
             print("\nPython example:\n\n")
-            print(f"from kodexa import *")
+            print(f"from kodexa import KodexaClient")
             print(f"client = KodexaClient('{KodexaPlatform.get_url()}', '{KodexaPlatform.get_access_token()}')")
     else:
         print("Kodexa is not logged in")
@@ -603,38 +657,59 @@ def version(_: Info):
 @click.option('--output', default=os.getcwd() + "/dist",
               help='Path to the output folder (defaults to dist under current)')
 @click.option('--version', default=os.getenv('VERSION'), help='Version number (defaults to 1.0.0)')
+@click.argument('files', nargs=-1, type=list[str])
 @pass_info
-def package(_: Info, path: str, output: str, version: str):
+def package(_: Info, path: str, output: str, version: str, files: list[str] = None, filename: Optional[str] = None):
     """
     Package an extension pack based on the kodexa.yml file
     """
-    metadata_obj = ExtensionHelper.load_metadata(path)
-    print("Preparing to pack")
-    try:
-        os.makedirs(output)
-    except OSError as e:
-        import errno
-        if e.errno != errno.EEXIST:
-            raise
 
-    metadata_obj['version'] = version if version is not None else '1.0.0'
+    if files is None:
+        files = ['kodexa.yml']
 
-    if 'source' in metadata_obj and 'location' in metadata_obj['source']:
-        metadata_obj['source']['location'] = metadata_obj['source']['location'].format(**metadata_obj)
+    for file in files:
+        metadata_obj = MetadataHelper.load_metadata(path, file)
 
-    versioned_metadata = os.path.join(output, f"{metadata_obj['slug']}-{metadata_obj['version']}.json")
+        print("Preparing to pack")
+        try:
+            os.makedirs(output)
+        except OSError as e:
+            import errno
+            if e.errno != errno.EEXIST:
+                raise
 
-    unversioned_metadata = os.path.join(output, "kodexa.json")
-    with open(versioned_metadata, 'w') as outfile:
-        json.dump(metadata_obj, outfile)
+        metadata_obj['version'] = version if version is not None else '1.0.0'
 
-    from shutil import copyfile
-    copyfile(versioned_metadata, unversioned_metadata)
+        def build_json():
+            versioned_metadata = os.path.join(output, f"{metadata_obj['slug']}-{metadata_obj['version']}.json")
+            unversioned_metadata = os.path.join(output, "kodexa.json")
+            with open(versioned_metadata, 'w') as outfile:
+                json.dump(metadata_obj, outfile)
 
-    output_filename = f"{metadata_obj['slug']}-{metadata_obj['version']}.tar.gz"
-    with tarfile.open(output_filename, "w:gz") as tar:
-        tar.add(output, arcname=os.path.basename(output))
+            copyfile(versioned_metadata, unversioned_metadata)
 
-    os.rename(output_filename, os.path.join(output, output_filename))
+        if 'type' not in metadata_obj:
+            metadata_obj['type'] = 'extensionPack'
 
-    print("Extension has been packaged :tada:")
+        if metadata_obj['type'] == 'extensionPack':
+            if 'source' in metadata_obj and 'location' in metadata_obj['source']:
+                metadata_obj['source']['location'] = metadata_obj['source']['location'].format(**metadata_obj)
+            build_json()
+
+            print("Extension pack has been packaged :tada:")
+
+        elif metadata_obj['type'].upper() == 'STORE' and metadata_obj['storeType'].upper() == 'MODEL':
+            model_content_metadata = ModelContentMetadata.parse_obj(metadata_obj['metadata'])
+            build_json()
+
+            # This will create the training.zip - we will then need to change the filename
+            ModelStoreEndpoint.build_implementation_zip(model_content_metadata)
+            versioned_implementation = os.path.join(output, f"{metadata_obj['slug']}-{metadata_obj['version']}.zip")
+            copyfile('implementation.zip', versioned_implementation)
+
+            # Delete the implementation
+            os.remove('implementation.zip')
+            print("Model has been packaged :tada:")
+
+        else:
+            print("We don't know how to package this type of metadata")
